@@ -109,7 +109,7 @@ class Action(object):
             logdir = self.session.config.logdir
         try:
             l = logdir.listdir("%s-*" % actionid)
-        except py.error.ENOENT:
+        except (py.error.ENOENT, py.error.ENOTDIR):
             logdir.ensure(dir=1)
             l = []
         num = len(l)
@@ -440,6 +440,12 @@ class Session:
             path.ensure(dir=1)
 
     def setupenv(self, venv):
+        if venv.envconfig.missing_subs:
+            venv.status = (
+                "unresolvable substitution(s): %s. "
+                "Environment variables are missing or defined recursively." %
+                (','.join(["'%s'" % m for m in venv.envconfig.missing_subs])))
+            return
         if not venv.matching_platform():
             venv.status = "platform mismatch"
             return  # we simply omit non-matching platforms
@@ -449,8 +455,17 @@ class Session:
             envlog = self.resultlog.get_envlog(venv.name)
             try:
                 status = venv.update(action=action)
-            except tox.exception.InvocationError:
-                status = sys.exc_info()[1]
+            except IOError as e:
+                if e.args[0] != 2:
+                    raise
+                status = (
+                    "Error creating virtualenv. Note that spaces in paths are "
+                    "not supported by virtualenv. Error details: %r" % e)
+            except tox.exception.InvocationError as e:
+                status = (
+                    "Error creating virtualenv. Note that some special "
+                    "characters (e.g. ':' and unicode symbols) in paths are "
+                    "not supported by virtualenv. Error details: %r" % e)
             if status:
                 commandlog = envlog.get_commandlog("setup")
                 commandlog.add_command(["setup virtualenv"], str(status), 1)
@@ -571,7 +586,7 @@ class Session:
             if venv.status:
                 return
             self.hook.tox_runtest_pre(venv=venv)
-            venv.test(redirect=redirect)
+            self.hook.tox_runtest(venv=venv, redirect=redirect)
             self.hook.tox_runtest_post(venv=venv)
         else:
             venv.status = "skipped tests"
@@ -645,7 +660,7 @@ class Session:
             self.report.line(msg)
         for e in default:
             report_env(e)
-        if all_envs:
+        if all_envs and extra:
             if description:
                 self.report.line('')
                 self.report.line('additional environments:')
